/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <efl_extension.h>
#include "$(appName).h"
#include "view.h"
#include "view_defines.h"

#define INDICATOR_HEIGHT 52
#define COLOR_DRAGGED_R 255
#define COLOR_DRAGGED_G 255
#define COLOR_DRAGGED_B 255
#define COLOR_DRAGGED_A 190
#define GRID_ITEM_SIZE_X 235
#define GRID_ITEM_SIZE_Y 180

typedef enum { POPUP_MAX_ITEMS = 0, POPUP_ITEM_EXIST } popup_type_t;
typedef enum { ITEM_TYPE_GRID = 0, ITEM_TYPE_LIST } item_type_t;

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *main_layout;
	Evas_Object *img_grid_layout;
	Evas_Object *img_grid;
	Evas_Object *image_preview;
	const char *image_under_preview_path;
	Elm_Gengrid_Item_Class *img_grid_itc;
	Evas_Object *src_object;
	Elm_Object_Item *src_item;
	Evas_Object *src_container;
	int src_idx;
	get_image_paths_cb get_paths_func;
	char *dragged_img_path;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.main_layout = NULL,
	.img_grid_layout = NULL,
	.img_grid = NULL,
	.image_preview = NULL,
	.image_under_preview_path = NULL,
	.img_grid_itc = NULL,
	.get_paths_func = NULL,
	.dragged_img_path = NULL,
};

static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *_get_grid_content_cb(void *data, Evas_Object *obj, const char *part);
static void _del_grid_item_cb(void *data, Evas_Object *obj);
static void _item_longpressed_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *_dnd_drag_icon_create_cb(void *data, Evas_Object *win, Evas_Coord *xoff, Evas_Coord *yoff);
static Eina_Bool _dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev);
static void _dnd_drag_done_cb(void *data, Evas_Object *obj);
static void _dnd_enter_cb(void *data, Evas_Object *obj);
static void _dnd_leave_cb(void *data, Evas_Object *obj);
static void _get_app_resource(const char *edj_file_in, char *edj_path_out);
static Eina_Bool _create_main_layout(void);
static Eina_Bool _create_image_slot(const char *part_name);
static Eina_Bool _create_collage_view(void);
static Eina_Bool _create_image_grid_view(void);
static bool _is_same_string(const char *s1, const char *s2);
static Eina_Bool _fill_gengrid(Evas_Object *gengrid);
static Evas_Object *_get_item_container_content(Elm_Object_Item *it);

/**
 * @brief Creates essential objects: window, conformant and layout.
 * @param[in] user_data The user data.
 * @return This function returns EINA_TRUE is the view was created successfully,
 * otherwise EINA_FALSE is returned.
 */
Eina_Bool view_create(void *user_data)
{
	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	s_info.conform = view_create_conformant(s_info.win);
	if (!s_info.conform)
		return EINA_FALSE;

	if (!_create_main_layout())
		return EINA_FALSE;

	if (!_create_collage_view())
		return EINA_FALSE;

	if (!_create_image_grid_view())
		return EINA_FALSE;

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window
 * @return This function returns window object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);

	/* Rotation setting */
	if (elm_win_wm_rotation_supported_get(win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(win, "delete,request", _delete_win_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a layout object and sets it to the default part of the parent object.
 * The layout object is created based on provided EDJE script.
 * @param[in] parent The parent object for layout object.
 * @param[in] edj_file_name The relative path to the layout EDJE script.
 * @param[in] edj_group The name of the group to be loaded from the EDJE script.
 * @return The function returns layout object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *edj_file_name, const char *edj_group)
{
	char edj_path[PATH_MAX] = {0, };
	Evas_Object *layout = NULL;

	if (!parent || !edj_file_name || !edj_group) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Wrong input arguments.");
		return NULL;
	}

	_get_app_resource(edj_file_name, edj_path);

	layout = elm_layout_add(parent);
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_layout_add() failed.");
		return NULL;
	}

	if (!elm_layout_file_set(layout, edj_path, edj_group)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_layout_file_set() failed.");
		evas_object_del(layout);
		return NULL;
	}

	eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates a conformant object for parent object.
 * @param[in] parent The parent object for conformant object.
 * @return The function returns conformant object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_conformant(Evas_Object *parent)
{
	Evas_Object *conform = NULL;
	Evas_Object *bg = NULL;

	conform = elm_conformant_add(parent);
	if (!conform) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_conformant_add() failed.");
		return NULL;
	}

	bg = elm_bg_add(conform);
	if (!bg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_bg_add() failed.");
		return NULL;
	}

	elm_bg_color_set(bg, 108, 102, 102);
	evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_layout_content_set(conform, "elm.swallow.indicator_bg", bg);
	evas_object_show(bg);

	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(parent, conform);
	evas_object_show(conform);

	return conform;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	elm_gengrid_item_class_free(s_info.img_grid_itc);
	evas_object_del(s_info.win);
}

/**
 * @brief Creates a layout object and sets it to the parent object based on provided EDJE script.
 * @param[in] parent The parent object for layout object.
 * @param[in] edj_file_name The relative path to the layout EDJE script.
 * @param[in] edj_group The name of the group to be loaded from the EDJE script.
 * @param[in] target_part_name The name of the EDJE part where the layout object has to be set.
 * @return The function returns layout object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_set_layout(Evas_Object *parent, const char *edj_file_name, const char *edj_group, const char *target_part_name)
{
	Evas_Object *layout = view_create_layout(parent, edj_file_name, edj_group);
	if (!layout)
		return NULL;

	if (!target_part_name)
		elm_object_content_set(parent, layout);
	else
		elm_layout_content_set(parent, target_part_name, layout);

	return layout;
}

/**
 * @brief Creates a gengrid object and sets it to the parent object.
 * @param[in] parent The parent object for gengrid object.
 * @param[in] target_part_name The name of the EDJE part where the gengrid object has to be set.
 * @return The function returns gengrid object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_gengrid(Evas_Object *parent, const char *target_part_name)
{
	Evas_Object *grid = NULL;

	if (!parent || !target_part_name) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Wrong input arguments.");
		return NULL;
	}

	grid = elm_gengrid_add(parent);
	if (!grid) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_gengrid_add() failed.");
		return NULL;
	}

	elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_NONE);
	elm_gengrid_horizontal_set(grid, EINA_TRUE);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_gengrid_item_size_set(grid, GRID_ITEM_SIZE_X, GRID_ITEM_SIZE_Y);
	elm_layout_content_set(parent, target_part_name, grid);
	evas_object_smart_callback_add(grid, "longpressed", _item_longpressed_cb, NULL);
	evas_object_show(grid);

	return grid;

}

/**
 * @brief Creates an image object using given path at given EDJE part.
 * @param[in] parent The parent object for image object.
 * @param[in] part_name The name of the EDJE part where the image object has to be set.
 * @return The function returns image object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_image(Evas_Object *parent, char *file_path, const char *part_name)
{
	Evas_Object *image = elm_image_add(parent);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_image_add() failed.");
		return NULL;
	}

	elm_image_smooth_set(image, EINA_TRUE);
	elm_image_file_set(image, file_path, NULL);

	if (part_name)
		elm_layout_content_set(parent, part_name, image);

	return image;
}

/**
 * @brief Creates an item class for gengrid object.
 * @param[in] on_text_get_cb The callback function's handler invoked when the gengrid's item has to be redrawn.
 * @return The function returns item class if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Elm_Gengrid_Item_Class *view_create_gengrid_item_class(Elm_Gengrid_Item_Content_Get_Cb on_content_get_cb, Elm_Gengrid_Item_Del_Cb on_item_del_cb)
{
	static Elm_Gengrid_Item_Class *itc = NULL;

	if (!itc) {
		itc = elm_gengrid_item_class_new();
		if (!itc) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_gengrid_item_class_new() failed.");
			return NULL;
		}

		itc->item_style = "default";
		itc->func.content_get = on_content_get_cb;
		itc->func.del = on_item_del_cb;
	}

	return itc;
}

/**
 * @brief Adds callback function handling data requests.
 * @param[in] get_paths_func The callback function invoked when the UI needs
 * to display images in the image browser area.
 */
void view_set_callback(get_image_paths_cb get_paths_func)
{
	s_info.get_paths_func = get_paths_func;
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_win_lower(s_info.win);
}

/**
 * @brief Internal callback function invoked on image grid's item display.
 * This function is assigned to the func.content_get handler of gengrid item's class structure.
 * In the content area of the gengrid's item an image is displayed.
 * @param[in] data The user data passed to the elm_gengrid_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The object to be displayed is returned.
 */
static Evas_Object *_get_grid_content_cb(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *layout;
	Evas_Object *image;
	if (_is_same_string(part, "elm.swallow.icon")) {
		layout = view_create_layout(obj, EDJ_IMG_GRID, GRP_IMG_GRID_ITEM);
		if (!layout)
			return NULL;

		image = view_create_image(layout, (char *)data, PART_IMG_GRID_ITEM_IMG);
		if (!image)
			return NULL;

		elm_image_aspect_fixed_set(image, EINA_TRUE);
		evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_show(layout);

		return layout;
	}
	return NULL;
}

/**
 * @brief Internal callback function invoked on image grid's item deletion.
 * This function is assigned to the func.del handler of gengrid item's class structure.
 * The image path attached to the list item is deleted here.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 */
static void _del_grid_item_cb(void *data, Evas_Object *obj)
{
	if (data)
		free(data);
}

/**
 * @brief Internal callback function invoked on long press over an image object.
 * This function initializes the drag-and-drop operation of the selected image.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_longpressed_cb(void *data, Evas_Object *obj, void *event_info)
{
	const char *file = NULL;
	Elm_Object_Item *it = (Elm_Object_Item *)event_info;
	Evas_Object *image = _get_item_container_content(it);

	elm_image_file_get(image, &file, NULL);
	if (!file) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_image_file_get() failed.");
		return;
	}

	elm_drag_start(image, ELM_SEL_FORMAT_TARGETS, file, ELM_XDND_ACTION_MOVE, _dnd_drag_icon_create_cb,
					image, NULL, NULL, NULL, NULL, _dnd_drag_done_cb, obj);

	elm_scroller_movement_block_set(obj, ELM_SCROLLER_MOVEMENT_BLOCK_HORIZONTAL);
}

/**
 * @brief The internal callback function invoked when the dragged object's icon
 * needs to be created.
 * @param[in] data The user data passed to the elm_drag_start() function.
 * @param[in] win The base window for drag icon.
 * @param[out] xoff The X offset at which to place the drag icon object in relation
 * to the source drag object.
 * @param[out] yoff The Y offset at which to place the drag icon object in relation
 * to the source drag object.
 * @return The function returns the drag icon if created successfully,
 * otherwise NULL is returned.
 */
static Evas_Object *_dnd_drag_icon_create_cb(void *data, Evas_Object *win, Evas_Coord *xoff, Evas_Coord *yoff)
{
	Evas_Object *org_ic = (Evas_Object *)data;
	Evas_Object *dragged_ic = NULL;
	const char *file = NULL;
	int x = -1;
	int y = -1;
	int w = -1;
	int h = -1;
	int xo = -1;
	int yo = -1;

	elm_image_file_get(org_ic, &file, NULL);
	dragged_ic = elm_image_add(win);
	s_info.dragged_img_path = (char *)file;

	if (!elm_image_file_set(dragged_ic, file, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_image_file_set() failed.");
		evas_object_del(dragged_ic);
		return NULL;
	}

	evas_object_geometry_get(org_ic, &x, &y, &w, &h);
	evas_pointer_canvas_xy_get(evas_object_evas_get(org_ic), &xo, &yo);

	evas_object_resize(dragged_ic, w / 2, h / 2);
	evas_object_move(dragged_ic, x - w / 2, y - h / 2);

	/* Setting offset to get center of moved icon behind mouse pointer. */
	if (xoff)
		*xoff = xo - w / 4;

	if (yoff)
		*yoff = yo - h / 4;

	return dragged_ic;
}

/**
 * @brief Internal callback function invoked by the image slot when dragged object enters target slot.
 * If there is an image, function unset and delete this image, then set preview on this image.
 * This callback function is attached with the elm_drop_target_add() function.
 * @param[in] data The user data passed to the elm_drop_target_add() function.
 * @param[in] obj The object invoking this callback function.
 */
static void _dnd_enter_cb(void *data, Evas_Object *obj)
{
	Evas_Object *image = elm_object_part_content_unset(obj, PART_MAIN_IMAGE_SLOT_CONTENT);
	elm_image_file_get(image, &s_info.image_under_preview_path, NULL);
	evas_object_del(image);

	s_info.image_preview = view_create_image(obj, s_info.dragged_img_path, PART_MAIN_IMAGE_SLOT_CONTENT);
	elm_image_fill_outside_set(s_info.image_preview, EINA_TRUE);
	evas_object_color_set(s_info.image_preview, 255, 255, 255, 80);
}

/**
 * @brief Internal callback function invoked by the image slot when dragged object leaves target slot.
 * Function unset and delete image preview, then if exist set previous image
 * This callback function is attached with the elm_drop_target_add() function.
 * @param[in] data The user data passed to the elm_drop_target_add() function.
 * @param[in] obj The object invoking this callback function.
 */
static void _dnd_leave_cb(void *data, Evas_Object *obj)
{
	Evas_Object *image = NULL;
	elm_object_part_content_unset(obj, PART_MAIN_IMAGE_SLOT_CONTENT);
	evas_object_del(s_info.image_preview);

	if (s_info.image_under_preview_path) {
		image = view_create_image(obj, (char*)s_info.image_under_preview_path, PART_MAIN_IMAGE_SLOT_CONTENT);
		elm_image_fill_outside_set(image, EINA_TRUE);
		s_info.image_under_preview_path = NULL;
	}
}

/**
 * @brief Internal callback function invoked by the collage (gengrid) on object drop.
 * Functions check position of drop, then set dropped image into desired image slot.
 * This callback function is attached with the elm_drop_target_add() function.
 * @param[in] data The user data passed to the elm_drop_target_add() function.
 * @param[in] obj The object invoking this callback function (gengrid).
 * @param[in] ev The event information structure.
 * @return The function returns 'EINA_TRUE' if the source object is acceptable and the drop operation
 * succeeds, otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev)
{
	const char *file = (char *)ev->data;
	Evas_Object *image;

	if (!file) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_dnd_drop_cb() failed. Image path is missing.");
		return EINA_FALSE;
	}

	image = view_create_image(obj, (char *)file, PART_MAIN_IMAGE_SLOT_CONTENT);
	s_info.image_under_preview_path = NULL;

	if (image)
		elm_image_fill_outside_set(image, EINA_TRUE);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on drag operation finish.
 * @param[in] data The user data passed to the elm_drag_start() function.
 * @param[in] obj The drag source object.
 * @param[in] event_info The information structure on event occurred.
 */
static void _dnd_drag_done_cb(void *data, Evas_Object *obj)
{
	Evas_Object *src_obj = (Evas_Object *)data;

	if (src_obj)
		elm_scroller_movement_block_set(src_obj, ELM_SCROLLER_MOVEMENT_NO_BLOCK);
}

/**
 * @brief Internal function which creates fully qualified path to the provided resource file.
 * @param[in] edj_file_in The file name.
 * @param[out] edj_path_out The fully qualified path to the edj_file_in file.
 */
static void _get_app_resource(const char *edj_file_in, char *edj_path_out)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(edj_path_out, PATH_MAX, "%s%s", res_path, edj_file_in);
		free(res_path);
	}
}

/**
 * @brief Internal function which creates the main panel (Space for collage and photos browser).
 * @return The function returns 'EINA_TRUE' if the main layout was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _create_main_layout(void)
{
	s_info.main_layout = view_create_set_layout(s_info.conform, EDJ_MAIN, GRP_MAIN, NULL);
	if (!s_info.main_layout)
		return EINA_FALSE;

	eext_object_event_callback_add(s_info.main_layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal function which create image slot for displaying image
 * @param[in] Part name where slot will be created
 * @return The function returns 'EINA_TRUE' if image slot was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _create_image_slot(const char *part_name)
{
	Evas_Object *layout = view_create_set_layout(s_info.main_layout, EDJ_MAIN, GRP_MAIN_IMAGE_SLOT, part_name);
	if (!layout)
		return EINA_FALSE;
	elm_drop_target_add(layout, ELM_SEL_FORMAT_TARGETS, _dnd_enter_cb, NULL, _dnd_leave_cb, NULL, NULL, NULL, _dnd_drop_cb, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal function which creates the layout for image collage.
 * @return The function returns 'EINA_TRUE' if the layout was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _create_collage_view(void)
{
	return (_create_image_slot(PART_MAIN_COLLAGE_IMG_0) &&
			_create_image_slot(PART_MAIN_COLLAGE_IMG_1) &&
			_create_image_slot(PART_MAIN_COLLAGE_IMG_2) &&
			_create_image_slot(PART_MAIN_COLLAGE_IMG_3) &&
			_create_image_slot(PART_MAIN_COLLAGE_IMG_4));
}

/**
 * @brief Internal function which creates the layout for image grid.
 * @return The function returns 'EINA_TRUE' if the layout was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _create_image_grid_view(void)
{
	Evas_Object *img_grid;
	Evas_Object *img_grid_layout = view_create_set_layout(s_info.main_layout, EDJ_IMG_GRID, GRP_IMG_GRID, PART_MAIN_IMG_GRID);
	if (!img_grid_layout)
		return EINA_FALSE;

	img_grid = view_create_gengrid(img_grid_layout, PART_IMG_GRID_CONTENT);
	if (!img_grid)
		return EINA_FALSE;

	s_info.img_grid_itc = view_create_gengrid_item_class(_get_grid_content_cb, _del_grid_item_cb);
	if (!s_info.img_grid_itc)
		return EINA_FALSE;

	if (!_fill_gengrid(img_grid))
		return EINA_FALSE;

	return EINA_TRUE;
}

/**
 * @brief Internal function which checks whether two strings are the same.
 * @param[in] s1 The first string.
 * @param[in] s2 The second string.
 * @return This function returns 'true' is both given strings are the same,
 * otherwise 'false' is returned.
 */
static bool _is_same_string(const char *s1, const char *s2)
{
	if (!s1 || !s2)
		return false;

	return (strncmp(s1, s2, strlen(s1)) == 0 && strlen(s1) == strlen(s2));
}

/**
 * @brief Internal function which gets the given container's content.
 * The function obtains the image object from the images of grid
 * item object.
 * @param[in] it The object item (item of gengrid).
 * @return The function returns the acquired image object, or NULL if failed.
 */
static Evas_Object *_get_item_container_content(Elm_Object_Item *it)
{
	Evas_Object *image = NULL;
	Evas_Object *layout = NULL;

	layout = elm_object_item_part_content_get(it, "elm.swallow.icon");
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_object_item_part_content_get() for item failed.");
		return NULL;
	}

	image = elm_object_part_content_get(layout, PART_IMG_GRID_ITEM_IMG);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_object_part_content_get() for image failed.");
		return NULL;
	}

	return image;
}

/**
 * @brief Internal function which populates the image grid with the images.
 * @param[in] gengrid The gengrid object of image grid area where the images
 * will be displayed.
 * @return The function returns 'EINA_TRUE' if the images browser is successfully populated,
 * with the images, otherwise 'EINA_FALSE' is returned.
 */
static Eina_Bool _fill_gengrid(Evas_Object *gengrid)
{
	Eina_List *img_paths = NULL;
	int i = 0;
	unsigned int count = 0;

	if (!s_info.get_paths_func) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_fill_gengrid() failed. Callback function not attached.");
		return EINA_FALSE;
	}

	s_info.get_paths_func(&img_paths);

	count = eina_list_count(img_paths);
	if (count == 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "The list of image paths is empty. No images to display.");
		eina_list_free(img_paths);
		return EINA_FALSE;
	}

	for (i = 0 ; i < count; i++)
		elm_gengrid_item_append(gengrid, s_info.img_grid_itc, eina_list_nth(img_paths, i), NULL, NULL);

	return EINA_TRUE;
}
